/*
 * Project name:
       MMC
 * Copyright:
       (c) Mikroelektronika, 2015.
 * Description:
       This module consists of several blocks that demonstrate various aspects of
       usage of the Mmc library. These are:
       - Creation of new file and writing down to it;
       - Opening existing file and re-writing it (writing from start-of-file);
       - Opening existing file and appending data to it (writing from end-of-file);
       - Opening a file and reading data from it (sending it to USART terminal);
       - Creating and modifying several files at once;
       - Reading file contents;
       - Deleting file(s);
       - Creating the swap file (see Help for details);

 * Test configuration:
         MCU:             FT900
                          http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
         Dev.Board:       Clicker 2 for FT900
                          http://www.mikroe.com/ft90x/clicker-2/
         Oscillator:      CPU at full system clock, 100.000MHz
         Ext. modules:    ac:MicroSD  - microSD click
                          ac:USB_Uart - USB UART click
         SW:              mikroC PRO for FT90x
                          http://www.mikroe.com/mikroc/FT90x/

 * NOTES:
       - Place USB Uart click board into the mikroBus socket 1.
       - Place microSD click board into the mikroBUS socket 2.
       - Plug USB cable in USB UART click.
       - Make sure that MMC card is properly formatted (to FAT16 or just FAT)
         before testing it on this example.
       - This example expects MMC card to be inserted before reset, otherwise,
         the FAT ERROR message is displayed.
 */

// MMC Chip Select connection
//**************************************************************************************************
sbit Mmc_Chip_Select           at GPIO_PIN33_bit;
//**************************************************************************************************

void InitMCU(){
  UART1_Init(56000);
  Delay_ms(1000);
}

const LINE_LEN = 43;
char err_txt[20];
char file_contents[LINE_LEN] = "XX MMC/SD FAT16 library by Anton Rieckert\n";
char           filename[14] = "MIKRO00x.TXT";          // File names
unsigned short loop, loop2;
unsigned long  i, size;
char           Buffer[512];

// UARTF write text and new line (carriage return + line feed)
void UART_Write_Line(char *uart_text) {
  UART_Write_Text(uart_text);
  UART_Write(13);
  UART_Write(10);
}

// Creates new file and writes some data to it
void M_Create_New_File() {
  filename[7] = 'A';
  Mmc_Fat_Set_File_Date(2010, 4, 19, 9, 0, 0); // Set file date & time info
  Mmc_Fat_Assign(&filename, 0xA0);          // Find existing file or create a new one
  Mmc_Fat_Rewrite();                        // To clear file and start with new data

  UART_Write(13);
  UART_Write(10);
  UART_Write_Text("Creating New File...");
  for(loop = 1; loop <= 99; loop++) {
    UART_Write('.');
    file_contents[0] = loop / 10 + 48;
    file_contents[1] = loop % 10 + 48;
    Mmc_Fat_Write(file_contents, LINE_LEN-1);   // write data to the assigned file
  }
}

// Creates many new files and writes data to them
void M_Create_Multiple_Files() {
  for(loop2 = 'B'; loop2 <= 'Z'; loop2++) {
    UART_Write(loop2);                  // signal the progress
    filename[7] = loop2;                 // set filename
    Mmc_Fat_Set_File_Date(2010, 4, 19, 9, 0, 0); // Set file date & time info
    Mmc_Fat_Assign(&filename, 0xA0);     // find existing file or create a new one
    Mmc_Fat_Rewrite();                   // To clear file and start with new data

    for(loop = 1; loop <= 44; loop++) {
      file_contents[0] = loop / 10 + 48;
      file_contents[1] = loop % 10 + 48;

      Mmc_Fat_Write(file_contents, LINE_LEN-1);  // write data to the assigned file
    }
  }
}

// Opens an existing file and rewrites it
void M_Open_File_Rewrite() {
  filename[7] = 'C';
  Mmc_Fat_Assign(&filename, 0);
  Mmc_Fat_Rewrite();

  UART_Write_Text("Open File Rewrite...");

  for(loop = 1; loop <= 55; loop++) {
    file_contents[0] = loop / 10 + 48;
    file_contents[1] = loop % 10 + 48;
    Mmc_Fat_Write(file_contents, LINE_LEN-1);    // write data to the assigned file
  }
}

// Opens an existing file and appends data to it
//               (and alters the date/time stamp)
void M_Open_File_Append() {
   filename[7] = 'B';
   Mmc_Fat_Assign(&filename, 0);
   Mmc_Fat_Set_File_Date(2010, 4, 19, 9, 20, 0);
   Mmc_Fat_Append();                                    // Prepare file for append
   Mmc_Fat_Write(" for mikroElektronika 2015\n", 27);   // Write data to assigned file
   UART_Write_Text("Open File Append...");
   UART_Write(13);
   UART_Write(10);
}

// Opens an existing file, reads data from it and puts it to UART
void M_Open_File_Read() {
  char character;

  filename[7] = 'B';
  Mmc_Fat_Assign(&filename, 0);
  Mmc_Fat_Reset(&size);            // To read file, procedure returns size of file
  for (i = 1; i <= size; i++) {
    Mmc_Fat_Read(&character);
    UART_Write(character);        // Write data to UART
  }
}

// Deletes a file. If file doesn't exist, it will first be created
// and then deleted.
void M_Delete_File() {
  filename[7] = 'F';
  Mmc_Fat_Assign(filename, 0);
  Mmc_Fat_Delete();
}

// Tests whether file exists, and if so sends its creation date
// and file size via UART
void M_Test_File_Exist() {
  unsigned long  fsize;
  unsigned int   year;
  unsigned short month, day, hour, minute;
  unsigned char  outstr[12];

  filename[7] = 'B';
//  filename[7] = 'F';
  if (Mmc_Fat_Assign(filename, 0)) {
    //--- file has been found - get its create date
    Mmc_Fat_Get_File_Date(&year, &month, &day, &hour, &minute);
    UART_Write_Text(" created: ");
    WordToStr(year, outstr);
    UART_Write_Text(outstr);
    ByteToStr(month, outstr);
    UART_Write_Text(outstr);
    WordToStr(day, outstr);
    UART_Write_Text(outstr);
    WordToStr(hour, outstr);
    UART_Write_Text(outstr);
    WordToStr(minute, outstr);
    UART_Write_Text(outstr);

    //--- file has been found - get its modified date
    Mmc_Fat_Get_File_Date_Modified(&year, &month, &day, &hour, &minute);
    UART_Write_Text(" modified: ");
    WordToStr(year, outstr);
    UART_Write_Text(outstr);
    ByteToStr(month, outstr);
    UART_Write_Text(outstr);
    WordToStr(day, outstr);
    UART_Write_Text(outstr);
    WordToStr(hour, outstr);
    UART_Write_Text(outstr);
    WordToStr(minute, outstr);
    UART_Write_Text(outstr);

    //--- get file size
    fsize = Mmc_Fat_Get_File_Size();
    LongToStr((signed long)fsize, outstr);
    UART_Write_Line(outstr);
  }
  else {
    //--- file was not found - signal it
    UART_Write(0x55);
    Delay_ms(1000);
    UART_Write(0x55);
  }
}
// Tries to create a swap file, whose size will be at least 100
// sectors (see Help for details)
void M_Create_Swap_File() {
  unsigned int i;

  for(i=0; i<512; i++)
    Buffer[i] = i;

  size = Mmc_Fat_Get_Swap_File(5000, "mikroE.txt", 0x20);   // see help on this function for details

  if (size) {
    LongToStr((signed long)size, err_txt);
    UART_Write_Line(err_txt);

    for(i=0; i<5000; i++) {
      Mmc_Write_Sector(size++, Buffer);
      UART_Write('.');
    }
  }
}

// Main. Uncomment the function(s) to test the desired operation(s)
void main() {
  #define COMPLETE_EXAMPLE         // comment this line to make simpler/smaller example
  InitMCU();

  Delay_ms(10);

  UART_Write(13);
  UART_Write(10);
  UART_Write_Line("1. FT900-Started");
  UART_Write_Line("2. Check for MMC");

  // Initialize SPI module
  SPIM1_Init_Advanced(_SPI_MASTER_CLK_RATIO_64, _SPI_CFG_POLARITY_IDLE_LOW | _SPI_CFG_PHASE_CAPTURE_FALLING, _SPI_SS_LINE_NONE);

  // use fat16 quick format instead of init routine if a formatting is needed
  if (Mmc_Fat_Init() == 0) {
    UART_Write_Line("3. MMC is ready to use");
    // reinitialize spi at higher speed
    SPIM1_Init_Advanced(_SPI_MASTER_CLK_RATIO_2, _SPI_CFG_POLARITY_IDLE_LOW | _SPI_CFG_PHASE_CAPTURE_FALLING, _SPI_SS_LINE_NONE);

    //--- Test start
    UART_Write_Line("4. Test Start:");
    //--- Test routines. Uncomment them one-by-one to test certain features
    M_Create_New_File();
    #ifdef COMPLETE_EXAMPLE
      M_Create_Multiple_Files();
      M_Open_File_Rewrite();
      M_Open_File_Append();
      M_Open_File_Read();
      M_Delete_File();
      M_Test_File_Exist();
      M_Create_Swap_File();
    #endif
    UART_Write(13);
    UART_Write(10);
    UART_Write(13);
    UART_Write(10);
    UART_Write_Line("5. Test End");
  }
  else
  {
    UART_Write_Line("3. FAT16 not found");
  }
}